<?php

/**
 *  Copyright (c) 2009 Dirla, tarent GmbH
 *
 *    Written by Axel Dirla <axel.dirla@tarent.de>
 **/

class IMAP_annotatemore{

    /**
     * Annotation Cache to reduce queries
     *
     * @var array
     */
    var $_annotation_cache;
 
    /**
     * IMAP server host.
     *
     * @var string
     */
    var $_server;

    /**
     * IMAP server port.
     *
     * @var string
     */
    var $_port;

    /**
     * Basic IMAP connection string.
     *
     * @var string
     */
    var $_base_mbox;

    /**
     * IMAP connection string that includes the folder.
     *
     * @var string
     */
    var $_mbox;

    /**
     * The signature of the current connection.
     *
     * @var string
     */
    var $_signature;

    /**
     * Copy of the current signature.
     *
     * @var string
     */
    var $_reuse_detection;    
    
    /**
     * IMAP user name.
     *
     * @var string
     */
    var $_login;

    /**
     * IMAP password.
     *
     * @var string
     */
    var $_password;

    /**
     * Constructor
     * Sets server host name and port
     * 
     * @param string server     The server host name
     * @param string port       The server port
     */
     function __construct( $server='localhost', $port='143' ){
       $this->_server = $server;
       $this->_port   = $port;
     }

    /**
     * Connects to the IMAP server.
     *
     * @param string $login     The user account name.
     * @param string $password  The user password.
     * @param boolean $tls      Should TLS be used for the connection?
     *
     * @return boolean          True in case the connection was opened
     *                             successfully.
     */
    function connect($login, $password, $tls = false, $cert = false)
    {
        $options = '';
        if (!$tls) {
            $options = '/notls';
        }else{
            $options = '/imap/ssl';
            if( !$cert ) $options = $options.'/novalidate-cert';
        }

        $mbox = '{' . $this->_server . ':' . $this->_port
            . $options . '}';

        $this->_signature = "$mbox|$login|$password";
        if ($this->_signature == $this->_reuse_detection) {
            return true;
        }

        $this->_mbox = $this->_base_mbox = $mbox;
        $this->_login = $login;
        $this->_password = $password;
        $this->_imap = null;

        $this->_reuse_detection = $this->_signature;

        return true;
    }

    /**
     * Lazy connect to the IMAP server.
     *
     * @return booleand  True in case the connection was opened successfully.
     */
    function _connect()
    {
        $result = @imap_open( $this->_base_mbox, $this->_login, 
                              $this->_password, OP_HALFOPEN );
        if (!$result) {
        	$err = sprintf( _("IMAP error on connect. Server: %s. Error: %s"), 
                            $this->_server, @imap_last_error()
                          ); 
            throw new Exception( $err );
        }
        $this->_imap = $result;
        return true;
    }

    /**
     * Disconnects from the IMAP server. If not really necessary this
     * should not be called. Once the page got served the connections
     * should be closed anyhow and if there is a chance to reuse the
     * connection it should be used.
     *
     * @return boolean  True in case the connection was closed successfully.
     */
    function disconnect()
    {
        if( isset($this->_imap) ){
          $this->_reuse_detection = null;

          $result = @imap_close($this->_imap);
          if (!$result) {
          	  $err=sprintf(_("IMAP error on disconnect. Server: %s. Error: %s"), 
          	               $this->_server, @imap_last_error()
          	               );
              throw new Exception( $err );
          }
          return $result;
        }
        return true;
    }



    /**
     * Retrieves a list of mailboxes on the server.
     *
     * @return mixed  The list of mailboxes.
     */
    function getMailboxes()
    {
        if (!isset($this->_imap)) {
            $result = $this->_connect();
        }

        $folders = array();

        $result = @imap_list($this->_imap, $this->_base_mbox, '*');
        if (!$result) {
        	$err = sprintf(_("IMAP error when retrieving list of mailboxes.".
        	                 " Folder: %s. Error: %s"), 
        	               $this->_base_mbox, @imap_last_error()
        	               );
            throw new Exception( $err );
        }

        $server_len = strlen($this->_base_mbox);
        foreach ($result as $folder) {
            if (substr($folder, 0, $server_len) == $this->_base_mbox) {
                $folders[] = substr($folder, $server_len);
            }
        }

        return $folders;
    }

    /**
     * Fetches the annotation on a folder.
     *
     * @param string $entry          The entry to fetch.
     * @param string $value          The specific value to fetch.
     * @param string $mailbox_name   The name of the folder.
     *
     * @return mixed                 The annotation value.
     */
    function getAnnotation($entry, $value, $mailbox_name)
    {
        if (!isset($this->_imap)) {
            $result = $this->_connect();
        }


        $signature = "$this->_signature|$entry|$value|$mailbox_name";

        if (!isset($this->_annotation_cache[$signature])) {
            $result = @imap_getannotation( $this->_imap, $mailbox_name, 
                                           $entry, $value );
            if( $result === false ){
              $err = sprintf(_("IMAP error when retrieving folder annotation.".
                               " Folder: %s. Error: %s"), 
                               $mailbox_name, @imap_last_error()
                               );
              throw new Exception( $err );
            }
            if (isset($result[$value])) {
                $this->_annotation_cache[$signature] = $result[$value];
            } else {
                $this->_annotation_cache[$signature] = '';
            }
        }

        return $this->_annotation_cache[$signature];
    }

    /**
     * Sets the annotation on a folder.
     *
     * @param string $entry          The entry to set.
     * @param array  $values         The values to set
     * @param string $mailbox_name   The name of the folder.
     *
     * @return boolean               True if successfull.
     */
    function setAnnotation($entry, $values, $mailbox_name)
    {
        if (!isset($this->_imap)) {
            $result = $this->_connect();
        }


        foreach ($values as $key => $value) {
            $result = @imap_setannotation( $this->_imap, $mailbox_name, $entry, 
                                           $key, $value );
            if (!$result) {
            	$err = sprintf(_("IMAP error when setting folder annotation.".
            	                 " Folder: %s. Error: %s"), 
            	               $mailbox_name, @imap_last_error()
            	               );
                throw new Exception( $err );
            }else{
                $signature = "$this->_signature|$entry|$key|$mailbox_name";
                if( strtoupper(trim($value)) == "NIL" ){
                  $this->_annotation_cache[$signature] = '';
                }else{
                  $this->_annotation_cache[$signature] = $value;
                }
            }
        }
        return true;
    }


}
